//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : uMain.pas
//Description : Example program for writing single values to Digital Out and Digital In/Out subdevices
//              - Open ME-iDS main driver
//              - determine subdevices of type Digital Out and Digital In/Out
//              - Add matched subdevices to the combobox
//              - Write value to selected subdevice when user clicks on button "Write Value"
//              - Close ME-iDS main driver, when program is closed
//==============================================================================
unit uMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Math, ExtCtrls, ComCtrls, StrUtils, meIDS;

type
//-------------------------------------------------
{TDoSubdevice}
  TDoSubdevice = class(TCollectionItem)
  private
    FDeviceName: string;
    FDeviceIndex: Integer;
    FSubdeviceIndex: Integer;
    FNumOfChannels: Integer;
    FSubdeviceCaps: Integer;
    FSingleConfigFlag: Integer;
    FSingleTypeFlag: Integer;
  public
    property DeviceName: string read FDeviceName;
    property DeviceIndex: Integer read FDeviceIndex;
    property SubdeviceIndex: Integer read FSubdeviceIndex;
    property NumOfChannels: Integer read FNumOfChannels;
    property SubdeviceCaps: Integer read FSubdeviceCaps;
    property SingleConfigFlag: Integer read FSingleConfigFlag;
    property SingleTypeFlag: Integer read FSingleTypeFlag;
  end;

{TDoSubdevices}
  TDoSubdevices = class(TCollection)
  private
    function GetSubdevice(Index: Integer): TDoSubdevice;
  public
    function AddSubdevice(ADeviceName: string; ADeviceIndex: Integer;
                          ASubdeviceIndex: Integer; ANumOfChannels: Integer;
                          ASubdeviceCaps: Integer): TDoSubdevice;
    property DoSubdevice[Index: Integer]: TDoSubdevice read GetSubdevice; default;
  end;


//-------------------------------------------------
  TfrmMain = class(TForm)
    memoInfo: TMemo;
    btnClose: TButton;
    editHexVal: TEdit;
    lbl4: TLabel;
    btnClearHwInfo: TButton;
    coBoxSubdevs: TComboBox;
    lbl3: TLabel;
    btnWriteValue: TButton;
    lbl1: TLabel;
    grpBox1: TGroupBox;
    btnAbout: TButton;
    btnReset: TButton;
    coBoxPortConfiguration: TComboBox;
    lbl2: TLabel;
    btnWriteZero: TButton;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnWriteValueClick(Sender: TObject);
    procedure btnClearHwInfoClick(Sender: TObject);
    procedure coBoxSubdevsChange(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnResetClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnWriteZeroClick(Sender: TObject);
  private
    { Private-Deklarationen }
    openErrSuccess: Integer;
    doSubdevices: TDoSubdevices;
    lineNumber: Integer;
    function MeVersionToString(iVersion: Integer): string;
    procedure GetDoDioSubdevices(numDevices: Integer);
    procedure GetSubdevicesByType(iDevice: Integer; iType: Integer);
    function GetSubDevTypeName(subdeviceType: Integer): string;
    procedure AddDoSubdevice(devName: string; iDevice: Integer; iSubdevice: Integer; subdeviceType: Integer);
    procedure DisplayErrorMessage(ErrMsg: string; meError: Integer);
    procedure OpenMEiDSDriver();
    procedure CloseMEiDSDriver();
    procedure DisplayVersionLibrary();
    procedure DisplayVersionMainDriver;
    function GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
    procedure DisplaySubdevices();
    procedure WriteValue(doSubdevice: TDoSubdevice; valueToWrite: Integer);
    function WriteSingleValue(doSubdevice: TDoSubdevice; valueToWrite: Integer): Integer;
    procedure DisplayValue(doSubdevice: TDoSubdevice; valueToWrite: Integer);
    function CheckValueToWrite(valStr: string; numChannels: Integer): Integer;
    function IntToBin(val: Cardinal; dispDigits: Integer): string;
    function ConfigureSubdeviceForOutput(doSubdevice: TDoSubdevice): Integer;
    procedure FillPortConfiguration;
  public
    { Public-Deklarationen }
  end;

var
  frmMain: TfrmMain;

implementation

uses meError, uAbout;

{$R *.dfm}

const
  ME_IDS_NOTOPENED = -1;


//-------------------------------------------------
{TDoSubdevices}
function TDoSubdevices.GetSubdevice(Index: Integer): TDoSubdevice;
begin
  Result := TDoSubdevice(inherited GetItem(Index));
end;

function TDoSubdevices.AddSubdevice(ADeviceName: string; ADeviceIndex: Integer;
                      ASubdeviceIndex: Integer; ANumOfChannels: Integer;
                      ASubdeviceCaps: Integer): TDoSubdevice;
begin
  Result := inherited Add as TDoSubdevice;
  Result.FDeviceName := ADeviceName;
  Result.FDeviceIndex := ADeviceIndex;
  Result.FSubdeviceIndex := ASubdeviceIndex;
  Result.FNumOfChannels := ANumOfChannels;
  Result.FSubdeviceCaps := ASubdeviceCaps;

  case ANumOfChannels of
    1:
    begin
       Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_BIT;
       Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_BIT;
    end;
    8:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_BYTE;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_BYTE;
    end;
    16:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_WORD;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_WORD;
    end;
    24:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_DWORD;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_DWORD;
    end;
    32:
    begin
      Result.FSingleConfigFlag := ME_IO_SINGLE_CONFIG_DIO_DWORD;
      Result.FSingleTypeFlag := ME_IO_SINGLE_TYPE_DIO_DWORD;
    end;
  else
    Result.FSingleConfigFlag := ME_VALUE_INVALID;
    Result.FSingleTypeFlag := ME_VALUE_INVALID;
  end;
end;

//-------------------------------------------------
procedure TfrmMain.FormCreate(Sender: TObject);
begin
  openErrSuccess := ME_IDS_NOTOPENED;
  lineNumber := 0;
  doSubdevices := TDoSubdevices.Create(TDoSubdevice);
  FillPortConfiguration;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  OpenMEiDSDriver();
end;

procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CloseMEiDSDriver();
  doSubdevices.Free;
end;

procedure TfrmMain.OpenMEiDSDriver();
var
  numDevices: Integer;
begin
  memoInfo.Lines.Append('Opening Maindriver...');
  openErrSuccess := meOpen(ME_OPEN_NO_FLAGS);

  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    DisplayVersionLibrary();
    DisplayVersionMainDriver();
    memoInfo.Lines.Append('');

    meQueryNumberDevices(numDevices);
    memoInfo.Lines.Append('Number of devices found: ' + IntToStr(numDevices));

    GetDoDioSubdevices(numDevices);
  end else
  begin
    DisplayErrorMessage('Error opening Maindriver ', openErrSuccess);
  end;
end;

procedure TfrmMain.GetDoDioSubdevices(numDevices: Integer);
var
  meError: Integer;
  idxDevice: Integer;
  plugged: Integer;
begin
  if (openErrSuccess = ME_ERRNO_SUCCESS) and (numDevices > 0) then
  begin
    memoInfo.Lines.Add('Retrieving sub-devices of type ME_TYPE_DO (Digital Out) and ME_TYPE_DIO (Digital In/Out)...');
    for idxDevice := 0 to numDevices - 1 do
    begin
      meError := GetDevicePlugged(idxDevice, plugged);

      if (meError = ME_ERRNO_SUCCESS) and (plugged = ME_PLUGGED_IN) then
      begin
        GetSubdevicesByType(idxDevice, ME_TYPE_DO);
        GetSubdevicesByType(idxDevice, ME_TYPE_DIO);
      end else
      begin
        memoInfo.Lines.Append('Device ' + IntToStr(idxDevice) + ' not plugged in.');
      end;
    end;
  end;
  DisplaySubdevices;
end;

procedure TFrmMain.GetSubdevicesByType(iDevice: Integer; iType: Integer);
var
  idxSubDevice: Integer;
  numSubDevices: Integer;
  meError: Integer;
  idxMatchedSubdevice: Integer;
  devName: array[0..ME_DEVICE_NAME_MAX_COUNT] of AnsiChar;
begin
  meError := meQueryNumberSubdevices(iDevice, numSubDevices);
  if meError = ME_ERRNO_SUCCESS then
  begin
    meQueryNameDevice(iDevice, devName, ME_DEVICE_NAME_MAX_COUNT);
    idxSubDevice := 0;
    while idxSubDevice < numSubDevices do
    begin
      meError := meQuerySubdeviceByType(iDevice,
                                        idxSubDevice,
                                        iType,
                                        ME_SUBTYPE_ANY,
                                        idxMatchedSubdevice);

      if meError = ME_ERRNO_SUCCESS then
      begin
        AddDoSubdevice(string(devName), iDevice, idxMatchedSubdevice, iType);
        idxSubDevice := idxMatchedSubdevice + 1;
      end else
      begin
        Break;
      end;
    end;
  end;
end;

procedure TfrmMain.AddDoSubdevice(devName: string; iDevice: Integer; iSubdevice: Integer; subdeviceType: Integer);
var
  numOfChannels: Integer;
  subdevCaps: Integer;
begin
  meQueryNumberChannels(iDevice, iSubdevice, numOfChannels);
  meQuerySubdeviceCaps(iDevice, iSubdevice, subdevCaps);
  doSubdevices.AddSubdevice(devName,
                            iDevice,
                            iSubdevice,
                            numOfChannels,
                            subdevCaps);

  coBoxSubdevs.Items.Append(string(devName)
                            + ',  Device ' + IntToStr(iDevice)
                            + ',  Subdevice ' + IntToStr(iSubdevice)
                            + ',  Type: ' + GetSubDevTypeName(subdeviceType)
                            + ',  Channels: ' + IntToStr(numOfChannels));
end;


procedure TfrmMain.btnResetClick(Sender: TObject);
var
  meError: Integer;
  doSubdevice: TDoSubdevice;
begin
  if (coBoxSubdevs.ItemIndex >= 0) then
  begin
    doSubdevice := doSubdevices[coBoxSubdevs.ItemIndex];

    meError := meIOResetSubdevice(doSubdevice.DeviceIndex,
                                  doSubdevice.SubdeviceIndex,
                                  ME_IO_RESET_SUBDEVICE_NO_FLAGS);
    if meError = ME_ERRNO_SUCCESS then
    begin
      memoInfo.Lines.Add('Reset Device ' + IntToStr(doSubdevice.DeviceIndex) + ', Subdevice ' + IntToStr(doSubdevice.SubdeviceIndex));
    end else
    begin
      DisplayErrorMessage('meIOResetSubdevice - Error: ', meError);
    end;
  end;
end;

procedure TfrmMain.btnWriteZeroClick(Sender: TObject);
var
  doSubdevice: TDoSubdevice;
begin
  if coBoxSubdevs.ItemIndex >= 0 then
  begin
    doSubdevice := doSubdevices[coBoxSubdevs.ItemIndex];
    WriteValue(doSubdevice, 0);
  end;
end;

procedure TfrmMain.btnWriteValueClick(Sender: TObject);
var
  valueToWrite: Integer;
  doSubdevice: TDoSubdevice;
begin
  if coBoxSubdevs.ItemIndex >= 0 then
  begin
    doSubdevice := doSubdevices[coBoxSubdevs.ItemIndex];
    try
      valueToWrite := CheckValueToWrite(editHexVal.Text, doSubdevice.NumOfChannels);
      WriteValue(doSubdevice, valueToWrite);
    except
      on e: Exception do
      begin
        MessageDlg('Error: ' + e.Message, mtError, [mbOK], 0);
        editHexVal.SetFocus;
      end;
    end;
  end;
end;

procedure TfrmMain.WriteValue(doSubdevice: TDoSubdevice; valueToWrite: Integer);
var
  meError: Integer;
begin
  meError := ConfigureSubdeviceForOutput(doSubdevice);
  if meError = ME_ERRNO_SUCCESS then
  begin
    meError := WriteSingleValue(doSubdevice, valueToWrite);
    if meError = ME_ERRNO_SUCCESS then
    begin
      DisplayValue(doSubdevice, valueToWrite);
    end else
    begin
      DisplayErrorMessage('meIOSingle - Error: ', meError);
    end;
  end else
  begin
    DisplayErrorMessage('meIOSingleConfig - Error: ', meError);
  end;
end;

function TfrmMain.ConfigureSubdeviceForOutput(doSubdevice: TDoSubdevice): Integer;
var
  meError: Integer;
  singleConfig: Integer;
begin
  if (doSubdevice.SubdeviceCaps and ME_CAPS_DIO_SINK_SOURCE) = ME_CAPS_DIO_SINK_SOURCE then
  begin
    singleConfig := Integer(coBoxPortConfiguration.Items.Objects[coBoxPortConfiguration.ItemIndex]);
    meError := meIOSingleConfig(doSubdevice.DeviceIndex,
                                doSubdevice.SubdeviceIndex,
                                0,
                                singleConfig,
                                ME_REF_NONE,
                                ME_TRIG_CHAN_NONE,
                                ME_VALUE_NOT_USED,
                                ME_VALUE_NOT_USED,
                                doSubdevice.SingleConfigFlag);
  end else
  begin
    meError := meIOSingleConfig(doSubdevice.DeviceIndex,
                                doSubdevice.SubdeviceIndex,
                                0,
                                ME_SINGLE_CONFIG_DIO_OUTPUT,
                                ME_REF_NONE,
                                ME_TRIG_CHAN_DEFAULT,
                                ME_TRIG_TYPE_NONE,
                                ME_VALUE_NOT_USED,
                                doSubdevice.SingleConfigFlag);
  end;
  Result := meError;
end;

function TfrmMain.WriteSingleValue(doSubdevice: TDoSubdevice; valueToWrite: Integer): Integer;
var
  io_single: array[0..0] of TMeIOSingle;
begin
  io_single[0].iDevice:= doSubdevice.DeviceIndex;
  io_single[0].iSubdevice := doSubdevice.SubdeviceIndex;
  io_single[0].iChannel := 0;
  io_single[0].iDir := ME_DIR_OUTPUT;
  io_single[0].iValue := valueToWrite;
  io_single[0].iTimeOut := 0;
  io_single[0].iFlags := doSubdevice.SingleTypeFlag;
  io_single[0].iErrno := 0;

  Result := meIOSingle(io_single[0],
                        1,
                        ME_IO_SINGLE_NO_FLAGS);
end;

procedure TfrmMain.DisplayValue(doSubdevice: TDoSubdevice; valueToWrite: Integer);
begin
  inc(lineNumber);
  memoInfo.lines.add(Format('%.2d', [lineNumber])
                        + ' Write to ' + doSubdevice.deviceName
                        + ',  Device: ' + IntToStr(doSubdevice.DeviceIndex)
                        + ',  Subdevice: ' + IntToStr(doSubdevice.SubdeviceIndex)
                        + ',  Value hex: ' + IntToHex(valueToWrite, doSubdevice.NumOfChannels div 4)
                        + ',  bin: ' + IntToBin(valueToWrite, doSubdevice.NumOfChannels)
                        + ',  dec: ' + IntToStr(valueToWrite));

end;

function TfrmMain.CheckValueToWrite(valStr: string; numChannels: Integer): Integer;
var
  x: Integer;
  m: Cardinal;
begin
  x := Cardinal(StrToInt('$' + valStr));
  if numChannels < 32 then
  begin
    m := (1 shl numChannels) -1;
    if Cardinal(x) > m then
    begin
      raise ERangeError.Create(Format('The value is out of range. Maximum value hex: %x', [m]));
    end;
    end;
  Result := x;
end;

procedure TfrmMain.coBoxSubdevsChange(Sender: TObject);
var
  doSubdevice: TDoSubdevice;
begin
  if coBoxSubdevs.ItemIndex >= 0 then
  begin
    doSubdevice := doSubdevices[coBoxSubdevs.ItemIndex];
    coBoxPortConfiguration.Enabled := (doSubdevice.SubdeviceCaps and ME_CAPS_DIO_SINK_SOURCE) = ME_CAPS_DIO_SINK_SOURCE;
  end;
end;

procedure TfrmMain.FillPortConfiguration;
begin
  coBoxPortConfiguration.AddItem('High impedance', TObject(ME_SINGLE_CONFIG_DIO_HIGH_IMPEDANCE));
  coBoxPortConfiguration.AddItem('Sink', TObject(ME_SINGLE_CONFIG_DIO_SINK));
  coBoxPortConfiguration.AddItem('Source', TObject(ME_SINGLE_CONFIG_DIO_SOURCE));
  coBoxPortConfiguration.ItemIndex := 0;
end;

procedure TfrmMain.DisplayErrorMessage(ErrMsg: string; meError: Integer);
var
  errorMsg: array[0..ME_ERROR_MSG_MAX_COUNT] of AnsiChar;
begin
  meErrorGetMessage(meError, errorMsg, ME_ERROR_MSG_MAX_COUNT);
  memoInfo.lines.add(ErrMsg + IntToStr(meError));
  memoInfo.Lines.Add('ErrorMessage: ' + errorMsg);
end;

procedure TfrmMain.CloseMEiDSDriver();
begin
  if openErrSuccess = ME_ERRNO_SUCCESS then
  begin
    meClose(ME_CLOSE_NO_FLAGS);
    openErrSuccess := ME_IDS_NOTOPENED;
    coBoxSubdevs.Clear;
    doSubdevices.Clear;
    btnWriteValue.Enabled := False;
    btnWriteZero.Enabled := False;
    btnReset.Enabled := False;
  end;
end;

procedure TfrmMain.DisplaySubdevices;
begin
  if coBoxSubdevs.Items.Count > 0 then
  begin
    memoInfo.Lines.Append('Number of DO and DIO sub-devices found: ' + IntToStr(coBoxSubdevs.Items.Count));
    memoInfo.Lines.AddStrings(coBoxSubdevs.Items);
    memoInfo.Lines.Add('');
    coBoxSubdevs.ItemIndex := 0;
    coBoxSubDevsChange(coBoxSubdevs);
    btnWriteValue.Enabled := True;
    btnWriteZero.Enabled := btnWriteValue.Enabled;
    btnReset.Enabled := btnWriteValue.Enabled;
  end else
  begin
    memoInfo.Lines.Append('No DO or DIO sub-devices found.');
    CloseMEiDSDriver;
  end;
end;

procedure TfrmMain.DisplayVersionLibrary();
var
  ver: Integer;
begin
  meQueryVersionLibrary(ver);
  memoInfo.Lines.Append('Library version: ' + MeVersionToString(ver));
end;

procedure TfrmMain.DisplayVersionMainDriver;
var
  ver: Integer;
begin
  meQueryVersionMainDriver(ver);
  memoInfo.Lines.Append('Main driver version: ' + MeVersionToString(ver));
end;

function TfrmMain.GetDevicePlugged(idxDevice: Integer; var plugged: Integer): Integer;
var
  busNo: Integer;
  vendorId: Integer;
  deviceId: Integer;
  funcNo: Integer;
  busType: Integer;
  serialNo: Integer;
  devNo: Integer;
begin
  Result := meQueryInfoDevice(idxDevice,
                    vendorId,
                    deviceId,
                    serialNo,
                    busType,
                    busNo,
                    devNo,
                    funcNo,
                    plugged);
end;

procedure TfrmMain.btnClearHwInfoClick(Sender: TObject);
begin
  memoInfo.Clear;
  lineNumber := 0;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

function TfrmMain.GetSubDevTypeName(subdeviceType: Integer): string;
begin
  case subdeviceType of
    ME_TYPE_INVALID: Result := 'ME_TYPE_INVALID';
    ME_TYPE_AI: Result := 'ME_TYPE_AI';
    ME_TYPE_AO: Result := 'ME_TYPE_AO';
    ME_TYPE_DIO: Result := 'ME_TYPE_DIO';
    ME_TYPE_DO: Result := 'ME_TYPE_DO';
    ME_TYPE_DI: Result := 'ME_TYPE_DI';
    ME_TYPE_CTR: Result := 'ME_TYPE_CTR';
    ME_TYPE_EXT_IRQ: Result := 'ME_TYPE_EXT_IRQ';
    ME_TYPE_FIO: Result := 'ME_TYPE_FIO';
    ME_TYPE_FO: Result := 'ME_TYPE_FO';
    ME_TYPE_FI: Result := 'ME_TYPE_FI';
    ME_TYPE_FPGA: Result := 'ME_TYPE_FPGA';
  else
    Result := '$' + IntToHex(subdeviceType, 8);
  end;
end;

function TfrmMain.IntToBin(val: Cardinal; dispDigits: Integer): string;
var
  i: Integer;
  b: Integer;
  tmpStr: string;
const
  binStr: array[0..15] of string = ('0000', '0001', '0010', '0011',
                                    '0100', '0101', '0110', '0111',
                                    '1000', '1001', '1010', '1011',
                                    '1100', '1101', '1110', '1111');
begin
  tmpStr := '';
  if dispDigits in [1, 8, 16, 24, 32] then
  begin
    if dispDigits = 1 then
    begin
      tmpStr := IntToStr(val and 1);
    end else
    begin
      for i := (dispDigits shr 2) -1 downto 0 do
      begin
        b := (val shr (i shl 2)) and $0F;
        tmpStr := tmpStr + binStr[b];
      end;
    end;
  end;
  Result := tmpStr;
end;

function TfrmMain.MeVersionToString(iVersion: Integer): string;
begin
  Result := Format('%d.%d.%d.%d', [iVersion shr 24, $FF and (iVersion shr 16), $FF and (iVersion shr 8), $FF and iVersion]);
end;

procedure TfrmMain.btnAboutClick(Sender: TObject);
begin
  frmAbout := TfrmAbout.Create(Self);
  try
    frmAbout.PopupParent := Self;
    frmAbout.ShowModal;
  finally
    frmAbout.Free;
  end;
end;

end.
