﻿//==============================================================================
//Copyright(c), Meilhaus Electronic GmbH
//              http://www.meilhaus.de
//              support@meilhaus.de
//
//Source File : FormMain.cs
//Description : Example program for reading property values
//              - Open main driver (ME-iDS)
//              - Create the root node (ME-iDS system container) in a TreeView
//              - Go recursively through the property containers and display all properties found
//              - add the name and the value of the property to the node text 
//              - add property attributes depending on the data type of the property 
//              - Close main driver (ME-iDS) when program is closed
//              - Properties can only be read from devices that are "plugged in"
//==============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using meIDS_CS;

namespace MEiDSProperties
{
    public partial class frmMain : Form
    {
        private int openErrSuccess = Constants.ME_IDS_NOTOPENED;

        public frmMain()
        {
            InitializeComponent();
        }

        private void frmMain_Shown(object sender, EventArgs e)
        {
            try
            {
                openErrSuccess = meIDS.meOpen(meIDS.ME_OPEN_NO_FLAGS);          
                if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
                {
                    DisplayMEiDSProperties();
                }
            }
            catch (DllNotFoundException ex)
            {
                MessageBox.Show("The ME-iDS is not properly installed." + Environment.NewLine + ex.Message, "ME-iDS Properties");
            }
        }

        private void DisplayMEiDSProperties()
        {
            TreeNode treeNode = new TreeNode(meIDS.ME_CONTAINER_NAME_ROOT);
            tv1.Nodes.Add(treeNode);
            DisplayContainerProperties(treeNode, "\\");
            tv1.ExpandAll();
            tv1.SelectedNode = treeNode;
            tv1.Focus();
        }

        private void DisplayContainerProperties(TreeNode parentNode, string containerPath)
        {
            string propertyPath = containerPath + "\\" + meIDS.ME_ATTRIBUTE_NAME_NUMBER_OF_ELEMENTS;
            int numElements;
            int propTypeVal;
            StringBuilder propStrVal = new StringBuilder(Constants.PROPERTY_STRING_LEN);
            int meError = meIDS.mePropertyGetIntW(propertyPath, out numElements);             
            if (meError == meIDS.ME_ERRNO_SUCCESS)
            {
                for (int i = 0; i < numElements; i++)
                {
                    StringBuilder propName = new StringBuilder();
                    propName.Append(containerPath + "\\" + i);
                    meError = meIDS.mePropertyGetStringW(propName + "\\" + meIDS.ME_ATTRIBUTE_NAME_PROPERTY_NAME, propStrVal, propStrVal.Capacity);
                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                    {
                        StringBuilder propType = new StringBuilder();
                        propType.Append(containerPath + "\\" + i + "\\" + meIDS.ME_ATTRIBUTE_NAME_PROPERTY_TYPE);
                        meError = meIDS.mePropertyGetIntW(propType.ToString(), out propTypeVal);
                        if (meError == meIDS.ME_ERRNO_SUCCESS)
                        {
                            StringBuilder nodeText = new StringBuilder(propStrVal.ToString());
                            if (propTypeVal == meIDS.ME_PROPERTY_TYPE_CONTAINER)
                            {
                                nodeText.Append(" Type: Container");
                                TreeNode treeNode = new TreeNode(nodeText.ToString());
                                parentNode.Nodes.Add(treeNode);
                                DisplayContainerProperties(treeNode, containerPath + "\\" + propStrVal);
                            }
                            else 
                            {
                                int propAccType;
                                StringBuilder propAccess = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_PROPERTY_ACCESS);
                                meError = meIDS.mePropertyGetIntW(propAccess.ToString(), out propAccType);
                                if (meError == meIDS.ME_ERRNO_SUCCESS)
                                {
                                    nodeText.Append(GetAccessTypeString(propAccType));
                                }
                                
                                StringBuilder propValue = new StringBuilder(containerPath + "\\" + propStrVal);
                                if (propTypeVal == meIDS.ME_PROPERTY_TYPE_BOOL)
                                {
                                    int propBoolVal;
                                    nodeText.Append(", Type: Boolean");
                                    meError = meIDS.mePropertyGetIntW(propValue.ToString(), out propBoolVal);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(", Value: " + GetPropertyBoolString(propBoolVal));
                                    }
                                }
                                else if (propTypeVal == meIDS.ME_PROPERTY_TYPE_INT)
                                {
                                    int propIntVal;
                                    nodeText.Append(", Type: Integer");
                                    meError = meIDS.mePropertyGetIntW(propValue.ToString(), out propIntVal);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(string.Format(", Value: {0} (0x{0:X})", propIntVal, propIntVal));
                                    }

                                    if (propAccType == meIDS.ME_PROPERTY_ACCESS_READ_WRITE)
                                    {
                                        int propIntMin;
                                        int propIntMax;
                                        StringBuilder propIntMinVal = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_MIN_VALUE);
                                        meError = meIDS.mePropertyGetIntW(propIntMinVal.ToString(), out propIntMin);
                                        if (meError == meIDS.ME_ERRNO_SUCCESS)
                                        {
                                            nodeText.Append(", Minimum: " + propIntMin);
                                            StringBuilder propIntMaxVal = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_MAX_VALUE);
                                            meError = meIDS.mePropertyGetIntW(propIntMaxVal.ToString(), out propIntMax);
                                            if (meError == meIDS.ME_ERRNO_SUCCESS)
                                            {
                                                nodeText.Append(", Maximum: " + propIntMax);
                                            }
                                        }
                                    }
                                }
                                else if (propTypeVal == meIDS.ME_PROPERTY_TYPE_DOUBLE)
                                {
                                    double propDoubleVal;
                                    nodeText.Append(", Type: Double");
                                    meError = meIDS.mePropertyGetDoubleW(propValue.ToString(), out propDoubleVal);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(", Value: " + propDoubleVal);
                                    }
                                    if (propAccType == meIDS.ME_PROPERTY_ACCESS_READ_WRITE)
                                    {
                                        Double propDoubleMin;
                                        Double propDoubleMax;
                                        StringBuilder propDoubleMinVal = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_MIN_VALUE);
                                        meError = meIDS.mePropertyGetDoubleW(propDoubleMinVal.ToString(), out propDoubleMin);
                                        if (meError == meIDS.ME_ERRNO_SUCCESS)
                                        {
                                            nodeText.Append(", Minimum: " + propDoubleMin);
                                            StringBuilder propDoubleMaxVal = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_MAX_VALUE);
                                            meError = meIDS.mePropertyGetDoubleW(propDoubleMaxVal.ToString(), out propDoubleMax);
                                            if (meError == meIDS.ME_ERRNO_SUCCESS)
                                            {
                                                nodeText.Append(", Maximum: " + propDoubleMax);
                                            }
                                        }
                                    }
                                }
                                else if (propTypeVal == meIDS.ME_PROPERTY_TYPE_STRING)
                                {
                                    StringBuilder propStringVal = new StringBuilder(Constants.PROPERTY_STRING_LEN);
                                    nodeText.Append(", Type: String");
                                    meError = meIDS.mePropertyGetStringW(propValue.ToString(), propStringVal, propStringVal.Capacity);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(", Value: \"" + propStringVal + "\"");
                                    }

                                    int propIntLength;
                                    StringBuilder propStringLength = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_LENGTH);
                                    meError = meIDS.mePropertyGetIntW(propStringLength.ToString(), out propIntLength);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(", Length: " + propIntLength);
                                    }

                                    if (propAccType == meIDS.ME_PROPERTY_ACCESS_READ_WRITE)
                                    {
                                        int propIntMaxLength;
                                        StringBuilder propStringMaxLength = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_MAX_LENGTH);
                                        meError = meIDS.mePropertyGetIntW(propStringMaxLength.ToString(), out propIntMaxLength);
                                        if (meError == meIDS.ME_ERRNO_SUCCESS)
                                        {
                                            nodeText.Append(", MaxLength: " + propIntMaxLength);
                                        }
                                    }

                                }
                                else if (propTypeVal == meIDS.ME_PROPERTY_TYPE_DEFINE)
                                {
                                    StringBuilder propDefineVal = new StringBuilder(Constants.PROPERTY_STRING_LEN);
                                    nodeText.Append(", Type: Define");
                                    meError = meIDS.mePropertyGetStringW(propValue.ToString(), propDefineVal, propDefineVal.Capacity);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(", Value: " + propDefineVal);
                                    }

                                    int propIntNumSelections;
                                    StringBuilder propNumSelections = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_NUMBER_OF_SELECTIONS);
                                    meError = meIDS.mePropertyGetIntW(propNumSelections.ToString(), out propIntNumSelections);
                                    if (meError == meIDS.ME_ERRNO_SUCCESS)
                                    {
                                        nodeText.Append(string.Format(", Selections: {0} (", propIntNumSelections));
                                        for (int j = 0; j < propIntNumSelections; j++)
                                        {
                                            StringBuilder propSelectionVal = new StringBuilder(Constants.PROPERTY_STRING_LEN);
                                            StringBuilder propSelection = new StringBuilder(containerPath + "\\" + propStrVal + "\\" + meIDS.ME_ATTRIBUTE_NAME_PREFIX_SELECTION + j.ToString());
                                            meError = meIDS.mePropertyGetStringW(propSelection.ToString(), propSelectionVal, propSelectionVal.Capacity);
                                            if (meError == meIDS.ME_ERRNO_SUCCESS)
                                            {
                                                nodeText.Append(propSelectionVal);
                                                if (j < propIntNumSelections -1)
                                                {
                                                    nodeText.Append(", ");
                                                }
                                            }
                                        }
                                        nodeText.Append(")");
                                    }
                                }
                                TreeNode treeNode = new TreeNode(nodeText.ToString());
                                parentNode.Nodes.Add(treeNode);
                            }
                        }
                    }
                }
            }
        }

        private string GetPropertyBoolString(int propertyBoolValue)
        {
            string tmpStr;
            if (propertyBoolValue == meIDS.ME_PROPERTY_BOOLEAN_FALSE)
            {
                tmpStr = "False";
            }
            else if (propertyBoolValue == meIDS.ME_PROPERTY_BOOLEAN_TRUE)
            {
                tmpStr = "True";
            }
            else
            {
                tmpStr = string.Format("Error boolean value: 0x{0:X}", propertyBoolValue);
            }
            return tmpStr;
        }

        private string GetAccessTypeString(int propertyAccessType)
        {
            string tmpStr;
            if (propertyAccessType == meIDS.ME_PROPERTY_ACCESS_READ_WRITE)
            {
                tmpStr = ", Read/Write";
            }
            else if (propertyAccessType == meIDS.ME_PROPERTY_ACCESS_READ_ONLY)
            {
                tmpStr = ", Read only";
            }
            else
            {
                tmpStr = string.Format("Error access type: 0x{0:X}", propertyAccessType);
            }
            return tmpStr;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void btnAbout_Click(object sender, EventArgs e)
        {
            AboutBox1 aboutBox = new AboutBox1();
            aboutBox.ShowDialog();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (openErrSuccess == meIDS.ME_ERRNO_SUCCESS)
            {
                meIDS.meClose(meIDS.ME_CLOSE_NO_FLAGS);
            }
        }
    }


    static class Constants
    {
        public const int ME_IDS_NOTOPENED = -1;
        public const int PROPERTY_STRING_LEN = 256;
    }
}
