﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using meIDS_CS;
using meIDSTools;

namespace TestConsole_meUtilityPWM_CS
{
    class TestConsole_meUtilityPWM
    {
 	    struct SPWMDevice
	    {
		    public int	m_iDeviceIndex;

		    public int m_iNumberOfCounterSubdevices;

		    public int[] m_arrCounterSubdevices;
	    };

        static void Main(string[] args)
        {
            int i_number_of_devices = TestConsoleGeneric.TestConsoleMEIDSProlog();

            if (i_number_of_devices <= 0)
            {
                return;
            }

	        // ---------------------------------------------------------------------

            // Count the devices in the system which are candidates for PWM.
            // These must have at least three counter sub-devices.

            int i_number_of_pwm_devices = 0;

            int i_me_error = 0;

            for (int index_device = 0; index_device < i_number_of_devices; index_device++)
            {
                int i_vendor_id;
                int i_device_id;
                int i_serial_no;
                int i_bus_type;
                int i_bus_no;
                int i_dev_no;
                int i_func_no;
                int i_plugged;

                i_me_error = meIDS.meQueryInfoDevice(   index_device,			// Device index
                                                        out i_vendor_id,		// Vendor ID returned here				
                                                        out i_device_id,		// Device ID returned here
                                                        out i_serial_no,		// Serial number returned here
                                                        out i_bus_type,			// Bus type returned here
                                                        out i_bus_no,			// Bus number returned here
                                                        out i_dev_no,			// Bus device number returned here
                                                        out i_func_no,			// Bus function number returned here
                                                        out i_plugged       );	// Plugged status returned here

                if (i_plugged != meIDS.ME_PLUGGED_IN)
                {
                    continue;
                }

                int i_number_ctr_8254_subdevices = 0;

                int index_subdevice = 0;

                int i_number_of_subdevices;

                i_me_error = meIDS.meQueryNumberSubdevices( index_device,				    // Device index
                                                            out i_number_of_subdevices);	// Number of subdevices returned here

                while (true)
                {
                    i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					// Device
                                                                index_subdevice,				    // Begin search at this sub-device 
                                                                meIDS.ME_TYPE_CTR,					// Type of sub-device to search for
                                                                meIDS.ME_SUBTYPE_CTR_8254,			// Sub-type of sub-device to search for
                                                                out index_subdevice         );		// sub-device index returned here

                    if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
                    {
                        break;
                    }

                    if (++i_number_ctr_8254_subdevices >= 3)
                    {
                        ++i_number_of_pwm_devices;

                        break;
                    }

                    if (++index_subdevice >= i_number_of_subdevices)
                    {
                        break;
                    }
                }
            }

            if (i_number_of_pwm_devices == 0)
            {
                System.Console.Write("****    No device for PWM found in system    ****\n\n", i_me_error);

                System.Console.Write("Press any key to terminate\n\n");

                System.Console.ReadKey(true);

                meIDS.meClose(0);

                return;
            }

            SPWMDevice[] arrPWMDevices = new SPWMDevice[i_number_of_pwm_devices];

            for (int index_pwm_device = 0; index_pwm_device < i_number_of_pwm_devices; index_pwm_device++)
            {
                arrPWMDevices[index_pwm_device].m_arrCounterSubdevices = new int[3];
            }

	        i_number_of_pwm_devices = 0;

	        for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	        {
		        int i_vendor_id;
		        int i_device_id;
		        int i_serial_no;
		        int i_bus_type;
		        int i_bus_no;
		        int i_dev_no;
		        int i_func_no;
		        int i_plugged;
        		
                i_me_error = meIDS.meQueryInfoDevice(   index_device,			// Device index
                                                        out i_vendor_id,		// Vendor ID returned here				
                                                        out i_device_id,		// Device ID returned here
                                                        out i_serial_no,		// Serial number returned here
                                                        out i_bus_type,			// Bus type returned here
                                                        out i_bus_no,			// Bus number returned here
                                                        out i_dev_no,			// Bus device number returned here
                                                        out i_func_no,			// Bus function number returned here
                                                        out i_plugged       );	// Plugged status returned here

		        if(i_plugged != meIDS.ME_PLUGGED_IN)
		        {
			        continue;
		        }

                arrPWMDevices[i_number_of_pwm_devices].m_iDeviceIndex = index_device;

                arrPWMDevices[i_number_of_pwm_devices].m_iNumberOfCounterSubdevices = 0;
                
                int index_subdevice = 0;
        		
		        int i_number_of_subdevices;

		        i_me_error = meIDS.meQueryNumberSubdevices(	index_device,				    // Device index
												            out i_number_of_subdevices);	// Number of subdevices returned here
        		
		        while(true)
		        {
			        i_me_error = meIDS.meQuerySubdeviceByType(  index_device,					    // Device
											                    index_subdevice,				    // Begin search at this sub-device 
											                    meIDS.ME_TYPE_CTR,					// Type of sub-device to search for
											                    meIDS.ME_SUBTYPE_CTR_8254,			// Sub-type of sub-device to search for
											                    out index_subdevice	        );		// sub-device index returned here

			        if(i_me_error != meIDS.ME_ERRNO_SUCCESS)
			        {
				        break;
			        }

			        arrPWMDevices[i_number_of_pwm_devices].m_arrCounterSubdevices[ arrPWMDevices[i_number_of_pwm_devices].m_iNumberOfCounterSubdevices ] = index_subdevice;

                    if(++arrPWMDevices[i_number_of_pwm_devices].m_iNumberOfCounterSubdevices >= 3)
			        {
				        ++i_number_of_pwm_devices;

				        break;
			        }

                    if (++index_subdevice >= i_number_of_subdevices)
                    {
                        break;
                    }
                }
	        }

	        System.Console.Write("{0} devices found for PWM:\n\n", i_number_of_pwm_devices);

	        for(int index_device = 0; index_device < i_number_of_pwm_devices; index_device++)
	        {
		        System.Console.Write(	"{0:D}.   On Device: {1:D}, sub-devices {2:D}, {3:D} and {4:D} are 8254 counter sub-devices\n",
				                        index_device + 1,
				                        arrPWMDevices[index_device].m_iDeviceIndex,
				                        arrPWMDevices[index_device].m_arrCounterSubdevices[0],
				                        arrPWMDevices[index_device].m_arrCounterSubdevices[1],
				                        arrPWMDevices[index_device].m_arrCounterSubdevices[2]		                        );
	        }

	        System.Console.Write("\n\nPress any key to continue\n\n");

            System.Console.ReadKey(true);

	        while(true)
	        {
		        int i_list_index_device = 0;

                string read_line;

		        bool b_terminate = false;

		        while(true)
		        {
			        System.Console.Write("Choose a device and sub-devices for PWM:\n\n");

			        for(int index_device = 0; index_device < i_number_of_pwm_devices; index_device++)
			        {
                        System.Console.Write("{0:D}.   Device: {1:D}, sub-devices {2:D}, {3:D} and {4:D}\n",
						                        index_device + 1,
						                        arrPWMDevices[index_device].m_iDeviceIndex,
						                        arrPWMDevices[index_device].m_arrCounterSubdevices[0],
						                        arrPWMDevices[index_device].m_arrCounterSubdevices[1],
						                        arrPWMDevices[index_device].m_arrCounterSubdevices[2]		);
			        }

			        System.Console.Write("\nYour choice (1 - {0:D}) (RETURN to terminate): ", i_number_of_pwm_devices);  

                    read_line = System.Console.ReadLine();
                    
			        System.Console.Write("\n\n");
        			
			        if(read_line.Length == 0)
			        {
				        b_terminate = true;
        				
				        break;
			        }
        			
				    i_list_index_device = System.Convert.ToInt32(read_line);

			        if( (i_list_index_device < 1)||(i_list_index_device > i_number_of_pwm_devices) )
			        {
				        continue;
			        }

			        --i_list_index_device;

			        break;
		        }

                if(b_terminate)
		        {
			        break;
		        }

		        int i_ref = 0;

		        string sz_ref = "";

		        int i_iref_choice = 0;
        			
		        while(true)
		        {
			        System.Console.Write("Choose a value for 'iRef':\n\n");

			        System.Console.Write(	"1.   ME_REF_CTR_EXTERNAL\n" +
					                        "2.   ME_REF_CTR_INTERNAL_1MHZ\n" +
					                        "3.   ME_REF_CTR_INTERNAL_10MHZ\n" +
					                        "4.   ME_REF_CTR_PREVIOUS\n"		 ); 

			        System.Console.Write("\nYour choice (1 - 4) (RETURN to terminate): ");  

                    read_line = System.Console.ReadLine();

                    System.Console.Write("\n\n");
        			
                    if (read_line.Length == 0)
                    {
                        b_terminate = true;

                        break;
                    }

                    i_iref_choice = System.Convert.ToInt32(read_line);

			        if( (i_iref_choice < 1)||(i_iref_choice > 4) )
			        {
				        continue;
			        }

			        break;
		        }

		        if(b_terminate)
		        {
			        break;
		        }

		        switch(i_iref_choice)
		        {
		        case 1:

                    i_ref = meIDS.ME_REF_CTR_EXTERNAL;

			        sz_ref = "ME_REF_CTR_EXTERNAL";

			        break;

		        case 2:

                    i_ref = meIDS.ME_REF_CTR_INTERNAL_1MHZ;

			        sz_ref = "ME_REF_CTR_INTERNAL_1MHZ";

			        break;

		        case 3:

                    i_ref = meIDS.ME_REF_CTR_INTERNAL_10MHZ;

			        sz_ref = "ME_REF_CTR_INTERNAL_10MHZ";

			        break;

		        case 4:

                    i_ref = meIDS.ME_REF_CTR_PREVIOUS;

			        sz_ref = "ME_REF_CTR_PREVIOUS";

			        break;
		        }

		        int i_prescaler = 1;

		        while(true)
		        {
			        System.Console.Write("Please enter a pre-scaler between 1 and 65534 (RETURN to terminate): ");

                    read_line = System.Console.ReadLine();

                    System.Console.Write("\n\n");
        			
                    if (read_line.Length == 0)
                    {
                        b_terminate = true;

                        break;
                    }

                    i_prescaler = System.Convert.ToInt32(read_line);

			        if( (i_prescaler < 1)||(i_prescaler > 65534) )
			        {
				        continue;
			        }

			        break;
		        }

		        if(b_terminate)
		        {
			        break;
		        }

		        int i_duty_cycle = 1;

		        while(true)
		        {
			        System.Console.Write("Please enter a duty cycle between 1% and 99% (RETURN to terminate): ");

                    read_line = System.Console.ReadLine();

                    System.Console.Write("\n\n");
        			
                    if (read_line.Length == 0)
                    {
                        b_terminate = true;

                        break;
                    }

                    i_duty_cycle = System.Convert.ToInt32(read_line);

			        if( (i_duty_cycle < 1)||(i_duty_cycle > 99) )
			        {
				        continue;
			        }

			        break;
		        }

		        if(b_terminate)
		        {
			        break;
		        }

		        int i_flag = 0;

		        string sz_flag = "";

		        int i_flag_choice = 0;

		        while(true)
		        {
			        System.Console.Write("Choose a value for 'iREF':\n\n");

			        System.Console.Write(	"1.   ME_PWM_START_NO_FLAGS\n" +
					                        "2.   ME_PWM_START_CONNECT_INTERNAL\n"	); 

			        System.Console.Write("\nYour choice (1 - 2) (RETURN to terminate): ");  

                    read_line = System.Console.ReadLine();

                    System.Console.Write("\n\n");
        			
                    if (read_line.Length == 0)
                    {
                        b_terminate = true;

                        break;
                    }

                    i_flag_choice = System.Convert.ToInt32(read_line);

			        if( (i_flag_choice < 1)||(i_flag_choice > 2) )
			        {
				        continue;
			        }

			        break;
		        }

		        if(b_terminate)
		        {
			        break;
		        }

		        if(i_flag_choice == 1)
		        {
                    i_flag = meIDS.ME_PWM_START_NO_FLAGS;

                    sz_flag = "ME_PWM_START_NO_FLAGS";
		        }
		        else
		        {
                    i_flag = meIDS.ME_PWM_START_CONNECT_INTERNAL;

                    sz_flag = "ME_PWM_START_CONNECT_INTERNAL";
		        }

		        System.Console.Write(	"Press RETURN to terminate or any other key to call:\n\n" +
                                        "    meUtilityPWMStart( {0:D}      (iDevice)\n" +
                                        "                       {1:D}      (iSubdevice1)\n" +
                                        "                       {2:D}      (iSubdevice2)\n" +
                                        "                       {3:D}      (iSubdevice3)\n" +
                                        "                       {4}        (iRef)\n" +
                                        "                       {5:D}      (iPrescaler)\n" +
                                        "                       {6:D}      (iDutyCycle)\n" +
                                        "                       {7}   );   (iFlag)\n",
				                        arrPWMDevices[i_list_index_device].m_iDeviceIndex,
				                        arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0],
				                        arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[1],
				                        arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[2],
				                        sz_ref,
				                        i_prescaler,
				                        i_duty_cycle,
				                        sz_flag													);

                System.Console.ReadKey(true);

                i_me_error = meIDS.meUtilityPWMStart(   arrPWMDevices[i_list_index_device].m_iDeviceIndex,
				                                        arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0],
				                                        arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[1],
				                                        arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[2],
				                                        i_ref,
				                                        i_prescaler,
				                                        i_duty_cycle,
				                                        i_flag													);
                
                if (i_me_error != meIDS.ME_ERRNO_SUCCESS)
                {
                    System.Console.Write("\n\n****    meUtilityPWMStart - Error: {0:D}    ****\n\n", i_me_error);

                    System.Console.Write("Press any key to continue\n");

                    System.Console.ReadKey(true);

                    continue;
                }



                System.Console.Write(   "\n\nPWM Started!\n\n" +
		                                "Press any key to call:\n\n" +
                                        "    meUtilityPWMStop( {0:D}      (iDevice)\n" +
                                        "                      {1:D}      (iSubdevice1)\n",							
		                                arrPWMDevices[i_list_index_device].m_iDeviceIndex,
		                                arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0]	);

                System.Console.ReadKey(true);

                meIDS.meUtilityPWMStop( arrPWMDevices[i_list_index_device].m_iDeviceIndex,
					                    arrPWMDevices[i_list_index_device].m_arrCounterSubdevices[0]	);
        		
                System.Console.Write( "\n\nPWM Stopped!\n\n");
            }
        	
	        meIDS.meClose(0);

            System.Console.Write("Program completed successfully - Press any key to terminate\n\n");

            System.Console.ReadKey(true);
        	
	        return;
        }
    }
}
