// Con_meIODigitalIrqCallback.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#include <stdio.h>
#include <conio.h>

#include <memory.h>

extern "C"
{
#include "..\\medriver.h"
}

#include "..\\TestConsoleGeneric.h"

#ifdef WIN64

#pragma comment (lib, "..\\meIDSmain64.lib")

#else

#pragma comment (lib, "..\\meIDSmain.lib")

#endif

// Forward declaration for callback function

int _stdcall IrqCallback(int iDevice, int iSubdevice, int iChannel, 
							int iIrqCount, int iValue, void *pvContext, int iErrorCode);

int _tmain(int argc, _TCHAR* argv[])
{
	// Call generic console function in parent directory to 
	// open system and list the available devices and subdevices.

	int i_number_of_devices = TestConsoleMEIDSProlog();

	if(i_number_of_devices <= 0)
	{
		return(-1);
	}

	// ---------------------------------------------------------------------
	
	// Count the DI sub-devices in the system that support bit mask or bit pattern IRQ

	int i_number_digital_irq_subdevices = 0;

	int i_me_error;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_DI,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			if( (i_caps & (ME_CAPS_DIO_BIT_PATTERN_IRQ | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) ) != 0 )
			{
				++i_number_digital_irq_subdevices;
			}
			
			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
		
		index_subdevice = 0;
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_DIO,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			if( (i_caps & (ME_CAPS_DIO_BIT_PATTERN_IRQ | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) ) != 0 )
			{
				++i_number_digital_irq_subdevices;
			}
			
			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
		
		index_subdevice = 0;
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_DO,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			if( (i_caps & ME_CAPS_DIO_OVER_TEMP_IRQ ) == ME_CAPS_DIO_OVER_TEMP_IRQ )
			{
				++i_number_digital_irq_subdevices;
			}
			
			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	if(i_number_digital_irq_subdevices == 0)
	{
		printf("****    No digital sub-devices found in system that support user IRQ   ****\n\n");
		
		printf("Press any key to terminate\n");
		
		_getch();
		
		meClose(0);
		
		return -1;
	}

	// Make a list of the digital sub-devices found in system that support user IRQ 

	struct SIRQSource
	{
		int m_iType;

		int m_iEdge;
	};

	struct SDigitalIrqSubddevice
	{
		int	m_iDeviceIndex;

		int m_iSubdeviceIndex;

		int m_iType;

		int m_iWidth;

		int m_iNumberOfSources;

		SIRQSource m_arrSources[4];
	};

	SDigitalIrqSubddevice* arrSubdevices = new SDigitalIrqSubddevice[i_number_digital_irq_subdevices];

	i_number_digital_irq_subdevices = 0;

	for(int index_device = 0; index_device < i_number_of_devices; index_device++)
	{
		int i_vendor_id;
		int i_device_id;
		int i_serial_no;
		int i_bus_type;
		int i_bus_no;
		int i_dev_no;
		int i_func_no;
		int i_plugged;
		
		i_me_error = meQueryInfoDevice(	index_device,				// Device index
										&i_vendor_id,				// Vendor ID returned here - not required				
										&i_device_id,				// Device ID returned here - not required
										&i_serial_no,				// Serial number returned here - not required
										&i_bus_type,				// Bus type returned here - not required
										&i_bus_no,					// Bus number returned here - not required
										&i_dev_no,					// Bus device number returned here - not required
										&i_func_no,					// Bus function number returned here - not required
										&i_plugged				);	// Plugged status returned hee - not required

		if(i_plugged != ME_PLUGGED_IN)
		{
			continue;
		}

		int index_subdevice = 0;
		
		int i_number_of_subdevices;

		i_me_error = meQueryNumberSubdevices(	index_device,				// Device index
												&i_number_of_subdevices);	// Number of subdevices returned here
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_DI,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			if( (i_caps & (ME_CAPS_DIO_BIT_PATTERN_IRQ | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) ) != 0 )
			{
				arrSubdevices[i_number_digital_irq_subdevices].m_iDeviceIndex = index_device;

				arrSubdevices[i_number_digital_irq_subdevices].m_iSubdeviceIndex = index_subdevice;

				arrSubdevices[i_number_digital_irq_subdevices].m_iType = ME_TYPE_DI;

				int i_number_sources = 0;

				if( (i_caps & ME_CAPS_DIO_BIT_PATTERN_IRQ) == ME_CAPS_DIO_BIT_PATTERN_IRQ )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_PATTERN;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_VALUE_NOT_USED;

					++i_number_sources;
				}

				if( (i_caps & ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING) == ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_MASK;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_IRQ_EDGE_RISING;

					++i_number_sources;
				}

				if( (i_caps & ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING) == ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_MASK;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_IRQ_EDGE_FALLING;

					++i_number_sources;
				}

				if( (i_caps & ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) == ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_MASK;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_IRQ_EDGE_ANY;

					++i_number_sources;
				}

				int i_number_of_channels;

				i_me_error = meQueryNumberChannels(	index_device,					// Device
													index_subdevice,				// Begin search at this sub-device 
													&i_number_of_channels);			// Number of channels returned here

				arrSubdevices[i_number_digital_irq_subdevices].m_iWidth = i_number_of_channels; 

				arrSubdevices[i_number_digital_irq_subdevices].m_iNumberOfSources = i_number_sources; 

				++i_number_digital_irq_subdevices;
			}

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
		
		index_subdevice = 0;
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_DIO,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			if( (i_caps & (ME_CAPS_DIO_BIT_PATTERN_IRQ | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING | ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) ) != 0 )
			{
				arrSubdevices[i_number_digital_irq_subdevices].m_iDeviceIndex = index_device;

				arrSubdevices[i_number_digital_irq_subdevices].m_iSubdeviceIndex = index_subdevice;

				arrSubdevices[i_number_digital_irq_subdevices].m_iType = ME_TYPE_DIO;

				int i_number_sources = 0;

				if( (i_caps & ME_CAPS_DIO_BIT_PATTERN_IRQ) == ME_CAPS_DIO_BIT_PATTERN_IRQ )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_PATTERN;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_VALUE_NOT_USED;

					++i_number_sources;
				}

				if( (i_caps & ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING) == ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_RISING )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_MASK;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_IRQ_EDGE_RISING;

					++i_number_sources;
				}

				if( (i_caps & ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING) == ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_FALLING )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_MASK;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_IRQ_EDGE_FALLING;

					++i_number_sources;
				}

				if( (i_caps & ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY) == ME_CAPS_DIO_BIT_MASK_IRQ_EDGE_ANY )
				{
					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iType = ME_IRQ_SOURCE_DIO_MASK;

					arrSubdevices[i_number_digital_irq_subdevices].m_arrSources[i_number_sources].m_iEdge = ME_IRQ_EDGE_ANY;

					++i_number_sources;
				}

				int i_number_of_channels;

				i_me_error = meQueryNumberChannels(	index_device,					// Device
													index_subdevice,				// Begin search at this sub-device 
													&i_number_of_channels);			// Number of channels returned here

				arrSubdevices[i_number_digital_irq_subdevices].m_iWidth = i_number_of_channels; 

				arrSubdevices[i_number_digital_irq_subdevices].m_iNumberOfSources = i_number_sources; 

				++i_number_digital_irq_subdevices;
			}

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
		
		index_subdevice = 0;
		
		while(true)
		{
			i_me_error = meQuerySubdeviceByType(index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												ME_TYPE_DO,						// Type of sub-device to search for
												ME_SUBTYPE_ANY,					// Sub-type of sub-device to search for
												&index_subdevice	);			// sub-device index returned here

			if(i_me_error != ME_ERRNO_SUCCESS)
			{
				break;
			}

			int i_caps;
			
			i_me_error = meQuerySubdeviceCaps(	index_device,					// Device
												index_subdevice,				// Begin search at this sub-device 
												&i_caps				);			// sub-device caps returned here

			if( (i_caps & ME_CAPS_DIO_OVER_TEMP_IRQ ) == ME_CAPS_DIO_OVER_TEMP_IRQ )
			{
				arrSubdevices[i_number_digital_irq_subdevices].m_iDeviceIndex = index_device;

				arrSubdevices[i_number_digital_irq_subdevices].m_iSubdeviceIndex = index_subdevice;

				arrSubdevices[i_number_digital_irq_subdevices].m_iType = ME_TYPE_DO;

				++i_number_digital_irq_subdevices;
			}

			if(++index_subdevice >= i_number_of_subdevices)
			{
				break;
			}
		}
	}

	printf("%d digital sub-devices that support bit mask, bit pattern or excess temperature IRQ found:\n\n", i_number_digital_irq_subdevices);

	for(int index_subdevice = 0; index_subdevice < i_number_digital_irq_subdevices; index_subdevice++)
	{
		printf("%2d.   Device: %2d  Sub-device: %2d Supports ",
				index_subdevice + 1,
				arrSubdevices[index_subdevice].m_iDeviceIndex,
				arrSubdevices[index_subdevice].m_iSubdeviceIndex					);

		if( (arrSubdevices[index_subdevice].m_iType == ME_TYPE_DI)||(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO) )
		{
			for(int index_source = 0; index_source < arrSubdevices[index_subdevice].m_iNumberOfSources; index_source++)
			{
				if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_PATTERN)
				{
					printf("Pattern");
				}
				else
				{
					printf("Mask - ");

					if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_RISING)
					{
						printf("Rising edge");
					}
					else if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_FALLING)
					{
						printf("Falling edge");
					}
					else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_ANY)
					{
						printf("Any edge");
					}
				}

				if(index_source < arrSubdevices[index_subdevice].m_iNumberOfSources - 1)
				{
					printf(", ");
				}
				else
				{
					printf("\n");
				}
			}
		}
		else
		{
			printf("Excess temperature\n");
		}
	}

	printf("\n\nPress any key to continue\n\n");

	getch();

	char buffer[512];

	bool b_terminate = false;
	
	while(true)
	{
		int index_subdevice;

		if(i_number_digital_irq_subdevices > 1)
		{
			while(true)
			{
				printf("Choose Subdevice:\n\n");

				for(index_subdevice = 0; index_subdevice < i_number_digital_irq_subdevices; index_subdevice++)
				{
					printf("    %2d.   Device: %2d  Sub-device: %2d Supports ",
							index_subdevice + 1,
							arrSubdevices[index_subdevice].m_iDeviceIndex,
							arrSubdevices[index_subdevice].m_iSubdeviceIndex					);

					if( (arrSubdevices[index_subdevice].m_iType == ME_TYPE_DI)||(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO) )
					{
						for(int index_source = 0; index_source < arrSubdevices[index_subdevice].m_iNumberOfSources; index_source++)
						{
							if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_PATTERN)
							{
								printf("Pattern");
							}
							else
							{
								printf("Mask - ");

								if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_RISING)
								{
									printf("Rising edge");
								}
								else if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_FALLING)
								{
									printf("Falling edge");
								}
								else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_ANY)
								{
									printf("Any edge");
								}
							}

							if(index_source < arrSubdevices[index_subdevice].m_iNumberOfSources - 1)
							{
								printf(", ");
							}
							else
							{
								printf("\n");
							}
						}
					}
					else
					{
						printf("Excess temperature\n");
					}
				}

				printf("\nYour choice (1 - %2d) (RETURN to terminate): ", i_number_digital_irq_subdevices);  

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &index_subdevice) != 1 ) )
				{
					continue;
				}

				if( (index_subdevice < 1)||(index_subdevice > i_number_digital_irq_subdevices) )
				{
					continue;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			--index_subdevice;
		}
		else
		{
			index_subdevice = 0;
		}

		int index_source;

		if( ( (arrSubdevices[index_subdevice].m_iType == ME_TYPE_DI)||(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO) )&&(arrSubdevices[index_subdevice].m_iNumberOfSources  > 1  ) )
		{
			while(true)
			{
				printf("Choose Source:\n\n");

				for(index_source = 0; index_source < arrSubdevices[index_subdevice].m_iNumberOfSources; index_source++)
				{
					printf("    %2d.   ", index_source + 1);

					if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_PATTERN)
					{
						printf("Pattern\n");
					}
					else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_MASK)
					{
						printf("Mask - ");

						if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_RISING)
						{
							printf("Rising edge\n");
						}
						else if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_FALLING)
						{
							printf("Falling edge\n");
						}
						else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_ANY)
						{
							printf("Any edge\n");
						}
					}
				}

				printf("\nYour choice (1 - %2d) (RETURN to terminate): ", arrSubdevices[index_subdevice].m_iNumberOfSources);  

				gets(&buffer[0]);

				printf("\n\n");
				
				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%d", &index_source) != 1 ) )
				{
					continue;
				}

				if( (index_source < 1)||(index_source > arrSubdevices[index_subdevice].m_iNumberOfSources) )
				{
					continue;
				}

				break;
			}

			if(b_terminate)
			{
				break;
			}

			--index_source;
		}
		else
		{
			index_source = 0;
		}

		int irq_start_arg;

		if( (arrSubdevices[index_subdevice].m_iType == ME_TYPE_DI)||(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO) )
		{
			while(true)
			{
				if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_PATTERN)
				{
					printf("Bit Pattern (Hex) (RETURN to terminate): 0x"); 
				}
				else //  // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_MASK)
				{
					printf("Bit Mask (Hex, %d Bit) (RETURN to terminate): 0x", arrSubdevices[index_subdevice].m_iWidth); 
				}

				gets(&buffer[0]);

				printf("\n\n");

				if(buffer[0] == 0)
				{
					b_terminate = true;
					
					break;
				}
				
				if( (sscanf(&buffer[0], "%X", &irq_start_arg) != 1 ) )
				{
					continue;
				}

				break;
			}
		}

		if(b_terminate)
		{
			break;
		}

		if(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO)
		{
			int i_config_flags;

			if(arrSubdevices[index_subdevice].m_iWidth == 8)
			{
				i_config_flags = ME_IO_SINGLE_CONFIG_DIO_BYTE;
			}
			else // if(arrSubdevices[index_subdevice].m_iWidth == 16)
			{
				i_config_flags = ME_IO_SINGLE_CONFIG_DIO_WORD;
			}

			i_me_error = meIOSingleConfig(	arrSubdevices[index_subdevice].m_iDeviceIndex,		// Device index
											arrSubdevices[index_subdevice].m_iSubdeviceIndex,	// Subdevcie index
											0,													// Channel
											ME_SINGLE_CONFIG_DIO_INPUT,							// Single config
											ME_VALUE_NOT_USED,									// Reference 
											ME_TRIG_CHAN_DEFAULT,								// Trigger channel
											ME_TRIG_TYPE_NONE,									// Trigger type
											ME_VALUE_NOT_USED,									// Trigger edge
											i_config_flags									);	// Flags
		}

		i_me_error = meIOIrqSetCallback(arrSubdevices[index_subdevice].m_iDeviceIndex,					// Device index
										arrSubdevices[index_subdevice].m_iSubdeviceIndex,				// Subdevice index
										IrqCallback,													// Callback routine
										NULL,															// Callback contextz
										ME_IO_IRQ_SET_CALLBACK_NO_FLAGS						);			// Flags


		printf("Ready to monitor ");

		if( (arrSubdevices[index_subdevice].m_iType == ME_TYPE_DI)||(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO) )
		{
			if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_PATTERN)
			{
				printf("Bit pattern interrupts (Pattern; 0x%02X) ", irq_start_arg);
			}
			else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType == ME_IRQ_SOURCE_DIO_MASK)
			{
				printf("Bit change interrupts, ");

				if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_RISING)
				{
					if(arrSubdevices[index_subdevice].m_iWidth == 8)
					{
						printf("Rising edge (Mask:  0x%02X) ", irq_start_arg);
					}
					else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iWidth == 16)
					{
						printf("Rising edge (Mask:  0x%04X) ", irq_start_arg);
					}
				}
				else if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_FALLING)
				{
					if(arrSubdevices[index_subdevice].m_iWidth == 8)
					{
						printf("Falling edge (Mask:  0x%02X) ", irq_start_arg);
					}
					else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iWidth == 16)
					{
						printf("Falling edge (Mask:  0x%04X) ", irq_start_arg);
					}
				}
				else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge == ME_IRQ_EDGE_ANY)
				{
					if(arrSubdevices[index_subdevice].m_iWidth == 8)
					{
						printf("Any edge (Mask:  0x%02X) ", irq_start_arg);
					}
					else // if(arrSubdevices[index_subdevice].m_arrSources[index_source].m_iWidth == 16)
					{
						printf("Any edge (Mask:  0x%04X) ", irq_start_arg);
					}
				}
			}
		}
		else
		{
			printf("Excess temperature interrupts ");
		}

		printf("on Device: %d, Subdevice: %d\n\n", 
				arrSubdevices[index_subdevice].m_iDeviceIndex, 
				arrSubdevices[index_subdevice].m_iSubdeviceIndex);

		printf("Press any key to start monitoring digital interrupts\n\n");
			
		printf("Press the ESC key at any time to stop monitoring interrupts\n\n");
		
		_getch();
		
		if( (arrSubdevices[index_subdevice].m_iType == ME_TYPE_DI)||(arrSubdevices[index_subdevice].m_iType == ME_TYPE_DIO) )
		{
			i_me_error =  meIOIrqStart(	arrSubdevices[index_subdevice].m_iDeviceIndex,							// Device index
										arrSubdevices[index_subdevice].m_iSubdeviceIndex,						// Subdevice index
										0,																		// Channel index 	
										arrSubdevices[index_subdevice].m_arrSources[index_source].m_iType,		// IRQ source
										arrSubdevices[index_subdevice].m_arrSources[index_source].m_iEdge,		// Type of signal edge
										irq_start_arg,															// Additional argument - Not required here
										ME_IO_IRQ_START_NO_FLAGS											);	// Flags
		}
		else
		{
			i_me_error =  meIOIrqStart(	arrSubdevices[index_subdevice].m_iDeviceIndex,			// Device index
										arrSubdevices[index_subdevice].m_iSubdeviceIndex,		// Subdevice index
										0,														// Channel index 	
										ME_IRQ_SOURCE_DIO_OVER_TEMP,							// IRQ source
										ME_VALUE_NOT_USED,										// Type of signal edge
										ME_VALUE_NOT_USED,										// Additional argument - Not required here
										ME_IO_IRQ_START_NO_FLAGS							);	// Flags
		}

		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("****    meIOIrqStart - Error: %d    ****\n\n", i_me_error);

			printf("Press any key to terminate\n");
			
			_getch();
			
			meClose(0);
			
			return -1;
		}

		printf("Monitoring interrupts\n\n");

		while(true)
		{
			if( _kbhit() != 0 )
			{
				if(_getch() == 27)
				{
					break;
				}
			}
		}

		i_me_error = meIOIrqStop(	arrSubdevices[index_subdevice].m_iDeviceIndex,				// Device index
									arrSubdevices[index_subdevice].m_iSubdeviceIndex,			// Subdevice index
									0,															// Channel index 	
									ME_IO_IRQ_STOP_NO_FLAGS								);		// Flags

	
		if(i_me_error != ME_ERRNO_SUCCESS)
		{
			printf("****    meIOIrqStop - Error: %d    ****\n\n", i_me_error);

			printf("Press any key to terminate\n");
			
			_getch();
		
			meClose(0);
		
			return(-1);
		}

		printf("Stopped monitoring interrupts\n\n");

		printf("Press any key to continue or ESC to exit\n");

		if(getch() == 27)
		{
			break;
		}
	}

	// ---------------------------------------------------------------------
	

	meClose(0);
	
	printf("\n\nProgram completed successfully - Press any key to terminate\n\n");

	_getch();
	
	return 0;
}

int _stdcall IrqCallback(int iDevice, int iSubdevice, int iChannel, 
							int iIrqCount, int iValue, void *pvContext, int iErrorCode)
{
	if(iErrorCode == ME_ERRNO_SUCCESS)
	{
		printf("Interrupt: IRQ Count: %d  Value: %d\n", iIrqCount, iValue);
	}
	else
	{
		printf("****    IrqCallback - Error: %d    ****\n\n", iErrorCode);
	}

	return(0);
}

